@@ -60,8 +60,8 @@ import numpy as np
 from scipy import linalg
 
 from . import tensors, utils
-from .ops import Channel, Gate, Operation, Unitary
-from .qubits import Qubit, Qubits
+from .ops import Channel, Gate, Operation, UnitaryGate
+from .qubits import Qubit, Qubits, sorted_qubits
 from .states import Density, State
 from .stdgates import I, X, Y, Z
 
@@ -105,7 +105,7 @@ class Kraus(Operation):
         """Returns: Action of Kraus operators as a superoperator Channel"""
         qubits = self.qubits
         N = len(qubits)
-        ident = Unitary(np.eye(2 ** N), qubits).aschannel()
+        ident = UnitaryGate(np.eye(2 ** N), qubits).aschannel()
 
         tensors = [(op.aschannel() @ ident).tensor for op in self.operators]
         if self.weights is not None:
@@ -142,10 +142,9 @@ class Kraus(Operation):
         Raises:
             TypeError: If qubits cannot be sorted into unique order.
         """
-        qbs = [q for elem in self.operators for q in elem.qubits]  # gather
-        qbs = list(set(qbs))  # unique
-        qbs = sorted(qbs)  # sort
-        return tuple(qbs)
+        # gather, unique, and sort
+        qbs = sorted_qubits(list(q for elem in self.operators for q in elem.qubits))
+        return qbs
 
     @property
     def H(self) -> "Kraus":
@@ -183,7 +182,8 @@ class UnitaryMixture(Kraus):
     def asgate(self) -> Gate:
         """Return one of the composite Kraus operators at random with
         the appropriate weights"""
-        return np.random.choice(self.operators, p=self.weights)
+        n = np.random.choice(range(len(self.operators)), p=self.weights)
+        return self.operators[n]
 
     def run(self, ket: State) -> State:
         return self.asgate().run(ket)
@@ -213,8 +213,8 @@ class Damping(Kraus):
     """
 
     def __init__(self, prob: float, q0: Qubit) -> None:
-        kraus0 = Unitary([[1.0, 0.0], [0.0, np.sqrt(1 - prob)]], [q0])
-        kraus1 = Unitary([[0.0, np.sqrt(prob)], [0.0, 0.0]], [q0])
+        kraus0 = UnitaryGate([[1.0, 0.0], [0.0, np.sqrt(1 - prob)]], [q0])
+        kraus1 = UnitaryGate([[0.0, np.sqrt(prob)], [0.0, 0.0]], [q0])
         super().__init__([kraus0, kraus1])
 
 
@@ -259,7 +259,7 @@ def channel_to_kraus(chan: Channel) -> "Kraus":
     for i in range(2 ** (2 * N)):
         if not np.isclose(values[i], 0.0):
             mat = np.reshape(evecs[i], (2 ** N, 2 ** N)) * values[i]
-            g = Unitary(mat, qubits)
+            g = UnitaryGate(mat, qubits)
             ops.append(g)
 
     return Kraus(ops)
@@ -272,13 +272,13 @@ def kraus_iscomplete(kraus: Kraus) -> bool:
     qubits = kraus.qubits
     N = kraus.qubit_nb
 
-    ident = Unitary(np.eye(2 ** N), qubits)
+    ident = UnitaryGate(np.eye(2 ** N), qubits)
 
     tensors = [(op.H @ op @ ident).asoperator() for op in kraus.operators]
     tensors = [t * w for t, w in zip(tensors, kraus.weights)]
 
     tensor = reduce(np.add, tensors)
-    res = Unitary(tensor, qubits)
+    res = UnitaryGate(tensor, qubits)
 
     N = res.qubit_nb
     return np.allclose(res.asoperator(), np.eye(2 ** N))
