@@ -131,10 +131,9 @@ def test_implicit_state() -> None:
     ket = circ.run()  # Implicit state
     assert len(ket.qubits) == 2
 
-    with pytest.raises(TypeError):
-        # Should fail because qubits aren't sortable, so no standard ordering
-        circ += qf.YPow(1 / 2, "namedqubit")
-        circ.qubits
+    # Mixed qubits types are now fine.
+    circ += qf.YPow(1 / 2, "namedqubit")
+    circ.qubits
 
 
 def test_elements() -> None:
@@ -251,7 +250,7 @@ def test_phase_estimation_circuit_3() -> None:
 
     with pytest.raises(ValueError):
         # Gate and output qubits overlap
-        circ = qf.phase_estimation_circuit(gate, range(N + 1))
+        _ = qf.phase_estimation_circuit(gate, range(N + 1))
 
 
 def test_addition_circuit() -> None:
@@ -349,17 +348,6 @@ def test_graph_state_circuit() -> None:
     _ = qf.graph_state_circuit(graph)
 
 
-def test_circuit_flat() -> None:
-    circ0 = qf.Circuit([qf.X(0), qf.X(1)])
-    circ1 = qf.Circuit([qf.Y(0), qf.Y(1)])
-    circ2 = qf.Circuit([circ1, qf.Z(0), qf.Z(1)])
-    circ = qf.Circuit([circ0, circ2])
-
-    flat = qf.Circuit(circ.flat())
-    assert len(flat) == 6
-    assert flat[2].name == "Y"
-
-
 def test_circuit_params() -> None:
     circ = qf.Circuit()
     circ += qf.X(0) ** 0.3
@@ -370,3 +358,43 @@ def test_circuit_params() -> None:
 
     with pytest.raises(ValueError):
         _ = circ.param("theta")
+
+
+def test_circuit_qubits() -> None:
+    circ = qf.Circuit([qf.X(1), qf.Z(3)], qubits=[0, 1, 3, 2])
+    assert circ.qubit_nb == 4
+    assert circ.qubits == (0, 1, 3, 2)
+
+    with pytest.raises(ValueError):
+        _ = qf.Circuit([qf.X(1), qf.Z(4)], qubits=[0, 1, 3, 2])
+
+
+def test_circuit_flat() -> None:
+    circ0 = qf.Circuit(qf.X(0), qf.X(1))
+    circ1 = qf.Circuit(qf.Y(0), qf.Y(1))
+    circ2 = qf.Circuit(circ1, qf.Z(0), qf.Z(1))
+    circ = qf.Circuit(circ0, circ2)
+    assert len(circ) == 2
+
+    flat = qf.Circuit(circ.flat())
+    assert len(flat) == 6
+    assert flat[2].name == "Y"
+
+
+def test_circuit_flat_lists() -> None:
+    circ0 = qf.Circuit([qf.X(0), qf.X(1)])
+    circ1 = qf.Circuit([qf.Y(0), qf.Y(1)])
+    circ2 = qf.Circuit([circ1, qf.Z(0), qf.Z(1)])
+    circ = qf.Circuit([circ0, circ2])
+
+    flat = qf.Circuit(circ.flat())
+    assert len(flat) == 6
+    assert flat[2].name == "Y"
+
+
+def test_circuit_order() -> None:
+    circ0 = qf.Circuit([qf.X(0), qf.X(1)], qubits=[1, 0])
+    assert circ0.qubits == (1, 0)
+    assert circ0.H.qubits == (1, 0)
+    assert circ0.on("b", "a").qubits == ("b", "a")
+    assert circ0.rewire({0: "b", 1: "a"}).qubits == ("b", "a")
