@@ -1,4 +1,5 @@
-# Copyright 2020-, Gavin E. Crooks and contributors
+# Copyright 2019-, Gavin E. Crooks and contributors
+# Copyright 2016-2018, Rigetti Computing
 #
 # This source code is licensed under the Apache License, Version 2.0 found in
 # the LICENSE.txt file in the root directory of this source tree.
@@ -19,19 +20,22 @@ sequence. Circuits can contain any instance of the abstract quantum operation
 superclass, Operation, including other circuits.
 
 Quantum operations are immutable, and transformations of these operations return
-new copies. (An exception is the composite operation DAGCircuit.)
+new copies.
 
-The main types of Operation's are Gate, Unitary, StdGate, Channel, Circuit, DAGCircuit,
-and Pauli.
+The main types of Operation's are Gate, UnitaryGate, StdGate, Channel, Circuit,
+DAGCircuit, and Pauli.
 
 .. autoclass:: Operation
     :members:
 
 """
 
-from abc import ABC  # Abstract Base Class
+import inspect
+from abc import ABC, abstractmethod
 from copy import copy
+from functools import total_ordering
 from typing import (
+    TYPE_CHECKING,
     Any,
     ClassVar,
     Dict,
@@ -46,16 +50,33 @@ from typing import (
 )
 
 import numpy as np
+import scipy
 from scipy.linalg import fractional_matrix_power as matpow
 from scipy.linalg import logm
 
 from . import tensors, utils, var
 from .qubits import Qubit, Qubits
 from .states import Density, State
-from .tensors import QubitTensor, TensorLike
+from .tensors import QubitTensor
 from .var import Variable
 
-__all__ = ["Operation", "Gate", "StdGate", "Unitary", "Channel"]
+# standard workaround to avoid circular imports from type hints
+if TYPE_CHECKING:
+    from numpy.typing import ArrayLike  # pragma: no cover
+
+    from .paulialgebra import Pauli  # pragma: no cover
+    from .stdgates import StdGate  # noqa:F401  # pragma: no cover
+
+
+__all__ = [
+    "Operation",
+    "Gate",
+    "UnitaryGate",
+    "Channel",
+    "Unitary",
+    "OPERATIONS",
+    "GATES",
+]
 
 
 OperationType = TypeVar("OperationType", bound="Operation")
@@ -65,6 +86,21 @@ GateType = TypeVar("GateType", bound="Gate")
 """Generic type annotations for subtypes of Gate"""
 
 
+_EXCLUDED_OPERATIONS = set(
+    ["Operation", "Gate", "StdGate", "StdCtrlGate", "In", "Out", "NoWire"]
+)
+# Names of operations to exclude from registration. Includes (effectively) abstract base
+# classes and internal operations.
+
+
+OPERATIONS: Dict[str, Type["Operation"]] = {}
+"""All quantum operations (All non-abstract subclasses of Operation)"""
+
+GATES: Dict[str, Type["Gate"]] = {}
+"""All gates (All non-abstract subclasses of Gate)"""
+
+
+@total_ordering
 class Operation(ABC):
     """An operation on a quantum state. An element of a quantum circuit.
 
@@ -87,16 +123,17 @@ class Operation(ABC):
     cv_qubit_nb: ClassVar[int] = None
     """The number of qubits, for operations with a fixed number of qubits"""
 
-    cv_args: Tuple[str, ...] = ()
-    """The names of the parameters for this operation"""
+    cv_args: ClassVar[Optional[Tuple[str, ...]]] = None
+    """The names of the parameters for this operation (For operations with a fixed number
+    of float parameters)"""
 
-    _diagram_labels: ClassVar[Optional[Sequence[str]]] = None
-    """Override default labels for drawing text circuit diagrams.
-    See visualizations.circuit_to_diagram()"""
+    def __init_subclass__(cls) -> None:
+        # Note: The __init_subclass__ initializes all subclasses of a given class.
+        # see https://www.python.org/dev/peps/pep-0487/
 
-    _diagram_noline: ClassVar[bool] = False
-    """Override default to not draw a line between qubit wires for multi-qubit
-    operations. See visualizations.circuit_to_diagram()"""
+        name = cls.__name__
+        if name not in _EXCLUDED_OPERATIONS:
+            OPERATIONS[name] = cls
 
     def __init__(
         self,
@@ -146,13 +183,16 @@ class Operation(ABC):
         qubits = tuple(labels[q] for q in self.qubits)
         return self.on(*qubits)
 
-    def qubit_indices(self, qubits: Qubits) -> List[int]:
+    def qubit_indices(self, qubits: Qubits) -> Tuple[int, ...]:
         """Convert qubits to index positions.
 
         Raises:
             ValueError: If argument qubits are not found in operation's qubits
         """
-        return [self.qubits.index(q) for q in qubits]
+        try:
+            return tuple(self.qubits.index(q) for q in qubits)
+        except ValueError:
+            raise ValueError("Incommensurate qubits")
 
     @property
     def params(self) -> Tuple[Variable, ...]:
@@ -172,6 +212,7 @@ class Operation(ABC):
 
         return self._params[idx]
 
+    # rename? param_asfloat? Then use where needed.
     def float_param(self, name: str, subs: Mapping[str, float] = None) -> float:
         """Return a a named parameters of this Operation as a float.
 
@@ -184,7 +225,7 @@ class Operation(ABC):
         """
         return var.asfloat(self.param(name), subs)
 
-    def resolve(self, subs: Mapping[str, float]) -> "Operation":
+    def resolve(self: OperationType, subs: Mapping[str, float]) -> OperationType:
         """Resolve symbolic parameters"""
         # params = {k: var.asfloat(v, subs) for k, v in self.params.items()}
         op = copy(self)
@@ -224,13 +265,21 @@ class Operation(ABC):
             "This operation does not support Hermitian conjugate"
         )  # pragma: no cover
 
-    @utils.cached_property
+    @property
     def tensor(self) -> QubitTensor:
         """
         Returns the tensor representation of this operation (if possible)
         """
         raise NotImplementedError()
 
+    @property
+    def tensor_diagonal(self) -> QubitTensor:
+        """
+        Returns the diagonal of the tensor representation of this operation
+        (if possible)
+        """
+        raise NotImplementedError()
+
     def run(self, ket: State) -> State:
         """Apply the action of this operation upon a pure state"""
         raise NotImplementedError()
@@ -244,6 +293,7 @@ class Operation(ABC):
     def __lt__(self, other: Any) -> bool:
         return id(self) < id(other)
 
+    # TODO: rename? standardize?
     def specialize(self) -> "Operation":
         """For parameterized operations, return appropriate special cases
         for particular parameters. Else return the original Operation.
@@ -272,6 +322,24 @@ class Operation(ABC):
 
         return Circuit(self)._repr_html_()
 
+    def _diagram_labels_(self) -> List[str]:
+        """Labels for text-based circuit diagrams.
+
+
+        Multi-qubit operations should either return one label per
+        qubit (which are then connected with vertical lines) or a
+        single label, which will be replicated onto all qubits and
+        not connected with vertical lines.
+        """
+        N = self.qubit_nb
+        labels = [self.name] * N
+        if N != 1 and not self.cv_interchangeable:
+            # If not interchangeable, we have to label connections
+            for i in range(N):
+                labels[i] = labels[i] + "_%s" % i
+
+        return labels
+
 
 # End class Operation
 
@@ -294,19 +362,31 @@ class Gate(Operation):
         identity
         diagonal
         permutation
+        swap
         monomial
 
     A permutation matrix permutes states. It has a single '1' in each row and column.
     All other entries are zero.
 
+    A swap is a permutation matrix that permutes qubits.
+
     A monomial matrix is a product of a diagonal and a permutation matrix.
     Only 1 entry in each row and column is non-zero.
 
     """
-    # TODO: Add "swap" t roster of tensor structures
 
-    # Note: Circular import hell
-    from .paulialgebra import Pauli
+    def __init_subclass__(cls) -> None:
+        # Note: The __init_subclass__ initializes all subclasses of a given class.
+        # see https://www.python.org/dev/peps/pep-0487/
+
+        if inspect.isabstract(cls):
+            return  # pragma: no cover
+
+        super().__init_subclass__()
+
+        name = cls.__name__
+        if name not in _EXCLUDED_OPERATIONS:
+            GATES[name] = cls
 
     @property
     def hamiltonian(self) -> "Pauli":
@@ -345,33 +425,45 @@ class Gate(Operation):
     def __pow__(self, t: float) -> "Gate":
         """Return this gate raised to the given power."""
         matrix = matpow(self.asoperator(), t)
-        return Unitary(matrix, self.qubits)
+        return UnitaryGate(matrix, self.qubits)
 
     def permute(self, qubits: Qubits) -> "Gate":
         """Permute the order of the qubits"""
+        qubits = tuple(qubits)
         if self.qubits == qubits:
             return self
         if self.cv_interchangeable:
             return self.on(*qubits)
         tensor = tensors.permute(self.tensor, self.qubit_indices(qubits))
-        return Unitary(tensor, qubits)
+        return UnitaryGate(tensor, qubits)
 
     def asoperator(self) -> QubitTensor:
         """Return tensor with with qubit indices flattened"""
         return tensors.flatten(self.tensor, rank=2)
-        # N = self.qubit_nb
-        # return np.reshape(self.tensor, [2**N] * 2)
 
-    def su(self) -> "Unitary":
+    @property
+    @abstractmethod
+    def tensor(self) -> QubitTensor:
+        pass
+
+    @utils.cached_property
+    def tensor_diagonal(self) -> QubitTensor:
+        """
+        Returns the diagonal of the tensor representation of this operation
+        (if possible)
+        """
+        return tensors.asqutensor(np.diag(self.asoperator()))
+
+    def su(self) -> "UnitaryGate":
         """Convert gate tensor to the special unitary group."""
         rank = 2 ** self.qubit_nb
         U = self.asoperator()
         U /= np.linalg.det(U) ** (1 / rank)
-        return Unitary(U, self.qubits)
+        return UnitaryGate(U, self.qubits)
 
     @property
     def H(self) -> "Gate":
-        return Unitary(self.asoperator().conj().T, self.qubits)
+        return UnitaryGate(self.asoperator().conj().T, self.qubits)
 
     def __matmul__(self, other: "Gate") -> "Gate":
         """Apply the action of this gate upon another gate,
@@ -388,20 +480,25 @@ class Gate(Operation):
         gate1 = other
         indices = gate1.qubit_indices(gate0.qubits)
         tensor = tensors.tensormul(gate0.tensor, gate1.tensor, tuple(indices))
-        return Unitary(tensor, gate1.qubits)
+        return UnitaryGate(tensor, gate1.qubits)
 
     def run(self, ket: State) -> State:
         """Apply the action of this gate upon a state"""
-        if self.cv_tensor_structure == "identity":
-            return ket
-
         qubits = self.qubits
         indices = ket.qubit_indices(qubits)
+
+        if self.cv_tensor_structure == "identity":
+            return ket
+        elif self.cv_tensor_structure == "diagonal":
+            tensor = tensors.tensormul_diagonal(
+                self.tensor_diagonal, ket.tensor, tuple(indices)
+            )
+            return State(tensor, ket.qubits, ket.memory)
+
         tensor = tensors.tensormul(
             self.tensor,
             ket.tensor,
             tuple(indices),
-            self.cv_tensor_structure == "diagonal",
         )
         return State(tensor, ket.qubits, ket.memory)
 
@@ -414,78 +511,6 @@ class Gate(Operation):
     def specialize(self) -> "Gate":
         return self
 
-
-# End class Gate
-
-
-class Unitary(Gate):
-    """
-    A quantum logic gate specified by an explicit unitary operator.
-    """
-
-    def __init__(self, tensor: TensorLike, qubits: Qubits) -> None:
-
-        tensor = tensors.asqutensor(tensor)
-
-        N = np.ndim(tensor) // 2
-
-        if len(tuple(qubits)) != N:
-            raise ValueError("Wrong number of qubits for tensor")
-
-        super().__init__(qubits=qubits)
-        self._tensor = tensor
-
-    @utils.cached_property
-    def tensor(self) -> QubitTensor:
-        """Returns the tensor representation of gate operator"""
-        return self._tensor
-
-
-# End class Unitary
-
-
-# TODO: Move?
-class StdGate(Gate):
-    """
-    A standard gate. Standard gates have a name, a fixed number of real
-    parameters, and act upon a fixed number of qubits.
-
-    e.g. Rx(theta, q0), CNot(q0, q1), Can(tx, ty, tz, q0, q1, q2)
-
-    In the argument list, parameters are first, then qubits. Parameters
-    have type Variable (either a concrete floating point number, or a symbolic
-    expression), and qubits have type Qubit (Any hashable python type).
-    """
-
-    cv_stdgates: Dict[str, Type["StdGate"]] = {}
-    """List of all StdGate subclasses"""
-
-    def __init_subclass__(cls) -> None:
-        # Note: The __init_subclass__ initializes all subclasses of a given class.
-        # see https://www.python.org/dev/peps/pep-0487/
-
-        # Parse the Gate arguments and number of qubits from the arguments to __init__
-        # Convention is that qubit names start with "q", but arguments do not.
-        names = getattr(cls, "__init__").__annotations__.keys()
-        args = tuple(s for s in names if s[0] != "q" and s != "return")
-        qubit_nb = len(names) - len(args)
-        if "return" in names:
-            # For unknown reasons, "return" is often, but not always in names.
-            qubit_nb -= 1
-
-        cls.cv_args = args
-        cls.cv_qubit_nb = qubit_nb
-
-        cls.cv_stdgates[cls.__name__] = cls  # Subclass registration
-
-    def __repr__(self) -> str:
-        args: List[str] = []
-        args.extend(str(p) for p in self.params)
-        args.extend(str(qubit) for qubit in self.qubits)
-        fargs = ", ".join(args)
-
-        return f"{self.name}({fargs})"
-
     def __str__(self) -> str:
         def _param_format(obj: Any) -> str:
             if isinstance(obj, float):
@@ -504,32 +529,79 @@ class StdGate(Gate):
 
         return f"{self.name}{fparams}{fqubits}"
 
+    # TODO: Move logic elsewhere
     def decompose(self) -> Iterator["StdGate"]:
-        from .translate import TRANSLATORS, translation_source_gate
+        from .translate import TRANSLATIONS, translation_source_gate
 
-        # Terminal gates
+        # Terminal gates  # FIXME: Move elsewhere?
         if self.name in ("I", "Ph", "X", "Y", "Z", "XPow", "YPow", "ZPow", "CNot"):
-            yield self
+            yield self  # type: ignore
             return
 
-        for trans in TRANSLATORS.values():
+        # Reversed so we favor translations added latter.
+        for trans in reversed(TRANSLATIONS):
             from_gate = translation_source_gate(trans)
             if isinstance(self, from_gate):
                 yield from trans(self)
                 return
 
-        yield self  # fall back  # pragma: no cover
+        # If we don't know how to perform an analytic translation, resort to a
+        # numerical decomposition. Will fail for gates with symbolic parameters.
+        from .decompositions import quantum_shannon_decomposition
+
+        circ = quantum_shannon_decomposition(self)
+        for gate in circ:
+            yield from gate.decompose()  # type: ignore
 
 
-# End class StdGate
+# End class Gate
 
 
+class UnitaryGate(Gate):
+    """
+    A quantum logic gate specified by an explicit unitary operator.
+    """
+
+    def __init__(self, tensor: "ArrayLike", qubits: Qubits) -> None:
+        tensor = tensors.asqutensor(tensor)
+
+        N = np.ndim(tensor) // 2
+        if len(tuple(qubits)) != N:
+            raise ValueError("Wrong number of qubits for tensor")
+
+        super().__init__(qubits=qubits)
+        self._tensor = tensor
+
+    @classmethod
+    def from_gate(cls, gate: Gate) -> "UnitaryGate":
+        return cls(gate.tensor, gate.qubits)
+
+    @classmethod
+    def from_hamiltonian(cls, hamiltonian: "Pauli", qubits: Qubits) -> "UnitaryGate":
+        """Create a Unitary gate U from a Pauli operator H, U = exp(-i H)"""
+        op = hamiltonian.asoperator(qubits)
+        U = scipy.linalg.expm(-1j * op)
+        return cls(U, qubits)
+
+    @utils.cached_property
+    def tensor(self) -> QubitTensor:
+        """Returns the tensor representation of gate operator"""
+        return self._tensor
+
+
+# End class UnitaryGate
+
+# Deprecated. Renamed to UnitaryGate
+Unitary = UnitaryGate
+
+
+# FIXME, more like UnitaryGate, with a 'Channel' superclass that's like Gate
 class Channel(Operation):
     """A quantum channel"""
 
     def __init__(
         self,
-        tensor: TensorLike,
+        tensor: "ArrayLike",
         qubits: Qubits,
         params: Sequence[var.Variable] = None,
         name: str = None,  # FIXME
@@ -601,7 +673,7 @@ class Channel(Operation):
         return np.reshape(self.sharp.tensor, [2 ** (N * 2)] * 2)
 
     @classmethod
-    def from_choi(cls, tensor: TensorLike, qubits: Qubits) -> "Channel":
+    def from_choi(cls, tensor: "ArrayLike", qubits: Qubits) -> "Channel":
         """Return a Channel from a Choi matrix"""
         return cls(tensor, qubits).sharp
 
@@ -671,4 +743,5 @@ class Channel(Operation):
 
 # end class Channel
 
+
 # fin
