@@ -33,14 +33,28 @@ Actions on states
 
 from abc import ABC
 from math import sqrt
-from typing import Any, Dict, List, Mapping, TextIO, Tuple, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    List,
+    Mapping,
+    TextIO,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
 import opt_einsum
 
 from . import tensors, utils
-from .qubits import Qubit, Qubits
-from .tensors import QubitTensor, TensorLike
+from .qubits import Qubit, Qubits, sorted_qubits
+from .tensors import QubitTensor
+
+if TYPE_CHECKING:
+    from numpy.typing import ArrayLike  # pragma: no cover
+
 
 __all__ = [
     "State",
@@ -64,7 +78,7 @@ QuantumStateType = TypeVar("QuantumStateType", bound="QuantumState")
 
 class QuantumState(ABC):
     def __init__(
-        self, tensor: TensorLike, qubits: Qubits, memory: Mapping = None
+        self, tensor: "ArrayLike", qubits: Qubits, memory: Mapping = None
     ) -> None:
         """
         Abstract base class for representations of a quantum state.
@@ -108,7 +122,7 @@ class QuantumState(ABC):
     def replace(
         self: QuantumStateType,
         *,
-        tensor: TensorLike = None,
+        tensor: "ArrayLike" = None,
         qubits: Qubits = None,
         memory: Mapping = None,
     ) -> QuantumStateType:
@@ -143,17 +157,17 @@ class QuantumState(ABC):
         ordering isn't supplied, we put qubits in sorted order.
         """
         if qubits is None:
-            qubits = sorted(self.qubits)
+            qubits = sorted_qubits(self.qubits)
         tensor = tensors.permute(self.tensor, self.qubit_indices(qubits))
         return self.replace(tensor=tensor, qubits=qubits)
 
-    def qubit_indices(self, qubits: Qubits) -> List[int]:
+    def qubit_indices(self, qubits: Qubits) -> Tuple[int, ...]:
         """Convert qubits to index positions.
 
         Raises:
             ValueError: If argument qubits are not found in state qubits
         """
-        return [self.qubits.index(q) for q in qubits]
+        return tuple(self.qubits.index(q) for q in qubits)
 
     def norm(self) -> QubitTensor:
         """Return the state vector norm"""
@@ -169,7 +183,7 @@ class State(QuantumState):
     """
 
     def __init__(
-        self, tensor: TensorLike, qubits: Qubits = None, memory: Mapping = None
+        self, tensor: "ArrayLike", qubits: Qubits = None, memory: Mapping = None
     ) -> None:
         """Create a new State from a tensor of qubit amplitudes
 
@@ -213,7 +227,7 @@ class State(QuantumState):
         return res
 
     def expectation(
-        self, diag_hermitian: TensorLike, trials: int = None
+        self, diag_hermitian: "ArrayLike", trials: int = None
     ) -> QubitTensor:
         """Return the expectation of a measurement. Since we can only measure
         our computer in the computational basis, we only require the diagonal
@@ -376,7 +390,7 @@ class Density(QuantumState):
     """A density matrix representation of a mixed quantum state"""
 
     def __init__(
-        self, tensor: TensorLike, qubits: Qubits = None, memory: Mapping = None
+        self, tensor: "ArrayLike", qubits: Qubits = None, memory: Mapping = None
     ) -> None:
         tensor = tensors.asqutensor(tensor)
 
@@ -399,7 +413,7 @@ class Density(QuantumState):
 
     # TESTME
     def probabilities(self) -> QubitTensor:
-        """Returns: The state probabilities """
+        """Returns: The state probabilities"""
         prob = tensors.diag(self.tensor)
         return prob
 
