@@ -7,25 +7,28 @@
 QuantumFlow: Standard two qubit gates
 """
 
-from typing import Iterator, Union
+from typing import List
 
 import numpy as np
 
 from .. import tensors, utils, var
-from ..config import CONJ, CTRL, SQRT, SWAP_TARGET, TARGET
-from ..gates import unitary_from_hamiltonian
-from ..ops import StdGate
+from ..config import CTRL, SWAP_TARGET, TARGET
+from ..ops import UnitaryGate
 from ..paulialgebra import Pauli, sX, sY, sZ
 from ..qubits import Qubit
 from ..states import State
 from ..tensors import QubitTensor
 from ..utils import cached_property
 from ..var import PI, Variable
-from .stdgates_1q import S_H, V_H, H, I, S, V, X, Y, Z
+from .stdgates import StdCtrlGate, StdGate
+from .stdgates_1q import V_H, H, I, S, T, V, X, XPow, Y, YPow, Z, ZPow
+
+_H = H
 
 # 2 qubit gates, alphabetic order
 
 __all__ = (
+    "A",
     "B",
     "Barenco",
     "Can",
@@ -33,6 +36,8 @@ __all__ = (
     "CNot",
     "CNotPow",
     "CrossResonance",
+    "CS",
+    "CT",
     "CV",
     "CV_H",
     "CY",
@@ -56,6 +61,39 @@ __all__ = (
 )
 
 
+class A(StdGate):
+    """The A gate. A 2-qubit, 2-parameter gate that is locally
+    equivalent to Can(1/2, t, t)
+
+    Refs:
+        :cite:`Barkoutsos2018a`
+
+        :cite:`Gard2020a`
+    """
+
+    cv_hermitian = True
+
+    def __init__(self, theta: Variable, phi: Variable, q0: Qubit, q1: Qubit):
+        super().__init__(params=[theta, phi], qubits=[q0, q1])
+
+    @cached_property
+    def tensor(self) -> QubitTensor:
+        phi = self.float_param("phi")
+        theta = self.float_param("theta")
+
+        U = [
+            [1, 0, 0, 0],
+            [0, np.cos(theta), np.exp(1j * phi) * np.sin(theta), 0],
+            [0, np.exp(-1j * phi) * np.sin(theta), -np.cos(theta), 0],
+            [0, 0, 0, 1],
+        ]
+        return tensors.asqutensor(U)
+
+    @property
+    def H(self) -> "A":
+        return self
+
+
 class B(StdGate):
     """The B (Berkeley) gate. Equivalent to Can(-1/2, -1/4, 0)"""
 
@@ -100,8 +138,6 @@ class Barenco(StdGate):
         https://arxiv.org/pdf/quant-ph/9505016.pdf
     """
 
-    _diagram_labels = ["───" + CTRL + "───", "Barenco({phi}, {alpha}, {theta})"]
-
     # Note: parameter order as defined by original paper
     def __init__(
         self,
@@ -144,6 +180,9 @@ class Barenco(StdGate):
         phi = phi + PI
         return Barenco(phi, alpha, theta, *self.qubits)
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["───" + CTRL + "───", "Barenco({phi}, {alpha}, {theta})"]
+
 
 # TODO: Add references and explanation
 # DOCME: Comment on sign conventions.
@@ -218,7 +257,7 @@ class Can(StdGate):
 # end class Can
 
 
-class CH(StdGate):
+class CH(StdCtrlGate):
     r"""A controlled-Hadamard gate
 
     Equivalent to ``controlled_gate(H())`` and locally equivalent to
@@ -233,28 +272,12 @@ class CH(StdGate):
                 0 & 0 & \tfrac{1}{\sqrt{2}} & -\tfrac{1}{\sqrt{2}}
             \end{pmatrix}
     """
-    _diagram_labels = [CTRL, "H"]
+    cv_target = _H
+    cv_hermitian = True
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        q0, q1 = self.qubits
-        return H(q1).hamiltonian * (1 - sZ(q0)) / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        unitary = np.asarray(
-            [
-                [1, 0, 0, 0],
-                [0, 1, 0, 0],
-                [0, 0, 1 / np.sqrt(2), 1 / np.sqrt(2)],
-                [0, 0, 1 / np.sqrt(2), -1 / np.sqrt(2)],
-            ]
-        )
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CH":
         return self  # Hermitian
@@ -265,7 +288,7 @@ class CH(StdGate):
 # end class CH
 
 
-class CNot(StdGate):
+class CNot(StdCtrlGate):
     r"""A controlled-not gate
 
     Equivalent to ``controlled_gate(X())``, and
@@ -275,22 +298,13 @@ class CNot(StdGate):
         \text{CNot}() \equiv \begin{pmatrix} 1&0&0&0 \\ 0&1&0&0 \\
                                             0&0&0&1 \\ 0&0&1&0 \end{pmatrix}
     """
+    cv_target = X
+    cv_hermitian = True
     cv_tensor_structure = "permutation"
-    _diagram_labels = [CTRL, TARGET]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        q0, q1 = self.qubits
-        return X(q1).hamiltonian * (1 - sZ(q0)) / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        unitary = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, 1], [0, 0, 1, 0]]
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CNot":
         return self  # Hermitian
@@ -307,12 +321,16 @@ class CNot(StdGate):
         tensor[s11] = ket.tensor[s10]
         return State(tensor, ket.qubits, ket.memory)
 
+    def _diagram_labels_(self) -> List[str]:
+        return [CTRL, TARGET]
+
 
 # end class CNot
 
 
 # FIXME: matrix in docs looks wrong?
-class CNotPow(StdGate):
+# FIXME: docs, controlled_gate
+class CNotPow(StdCtrlGate):
     r"""Powers of the CNot gate.
 
     Equivalent to ``controlled_gate(TX(t))``, and locally equivalent to
@@ -334,31 +352,11 @@ class CNotPow(StdGate):
         q0: control qubit
         q1: target qubit
     """
-
-    _diagram_labels = [CTRL, "X^{t}"]
+    cv_target = XPow
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        (t,) = self.params
-        return CNot(*self.qubits).hamiltonian * t
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        ctheta = np.pi * var.asfloat(self.param("t"))
-        phase = np.exp(0.5j * ctheta)
-        cht = np.cos(ctheta / 2)
-        sht = np.sin(ctheta / 2)
-        unitary = [
-            [1, 0, 0, 0],
-            [0, 1, 0, 0],
-            [0, 0, phase * cht, phase * -1.0j * sht],
-            [0, 0, phase * -1.0j * sht, phase * cht],
-        ]
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CNotPow":
         return self ** -1
@@ -385,8 +383,6 @@ class CrossResonance(StdGate):
     # DOCME
     # TESTME
 
-    _diagram_labels = ["CR({s}, {b}, {c})_q0", "CR({s}, {b}, {c})_q1"]
-
     def __init__(self, s: Variable, b: Variable, c: Variable, q0: Qubit, q1: Qubit):
         super().__init__(qubits=(q0, q1), params=[s, b, c])
 
@@ -398,7 +394,7 @@ class CrossResonance(StdGate):
 
     @cached_property
     def tensor(self) -> QubitTensor:
-        U = unitary_from_hamiltonian(self.hamiltonian, self.qubits)
+        U = UnitaryGate.from_hamiltonian(self.hamiltonian, self.qubits)
         return tensors.asqutensor(U.tensor)
 
     @property
@@ -410,11 +406,56 @@ class CrossResonance(StdGate):
         s, b, c = self.params
         return CrossResonance(t * s, b, c, *self.qubits)
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["CR({s}, {b}, {c})_0", "CR({s}, {b}, {c})_1"]
+
 
 # end class CrossResonance
 
 
-class CY(StdGate):
+class CS(StdCtrlGate):
+    r"""A controlled-S gate
+
+    .. math::
+        \text{CS}() = \begin{pmatrix} 1&0&0&0 \\ 0&1&0&0 \\
+                                    0&0&1&0 \\ 0&0&0&i \end{pmatrix}
+    """
+    cv_target = S
+    cv_interchangeable = True
+    cv_tensor_structure = "diagonal"
+
+    def __init__(self, q0: Qubit, q1: Qubit) -> None:
+        super().__init__(qubits=[q0, q1])
+
+    def __pow__(self, t: Variable) -> "CZPow":
+        return CZPow(t / 2, *self.qubits)
+
+
+# End class CS
+
+
+class CT(StdCtrlGate):
+    r"""A controlled-T gate
+
+    .. math::
+        \text{CT}() = \begin{pmatrix} 1&0&0&0 \\ 0&1&0&0 \\
+                                    0&0&1&0 \\ 0&0&0&  \end{pmatrix}
+    """
+    cv_target = T
+    cv_interchangeable = True
+    cv_tensor_structure = "diagonal"
+
+    def __init__(self, q0: Qubit, q1: Qubit) -> None:
+        super().__init__(qubits=[q0, q1])
+
+    def __pow__(self, t: Variable) -> "CZPow":
+        return CZPow(t / 4, *self.qubits)
+
+
+# End class CS
+
+
+class CY(StdCtrlGate):
     r"""A controlled-Y gate
 
     Equivalent to ``controlled_gate(Y())`` and locally equivalent to
@@ -429,24 +470,13 @@ class CY(StdGate):
                 0 & 0 & i & 0
             \end{pmatrix}
     """
+    cv_target = Y
+    cv_hermitian = True
     cv_tensor_structure = "monomial"
-    _diagram_labels = [CTRL, "Y"]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        q0, q1 = self.qubits
-        return Y(q1).hamiltonian * (1 - sZ(q0)) / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        unitary = np.asarray(
-            [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, -1j], [0, 0, 1j, 0]]
-        )
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CY":
         return self  # Hermitian
@@ -458,34 +488,17 @@ class CY(StdGate):
 # end class CY
 
 
-class CYPow(StdGate):
+class CYPow(StdCtrlGate):
     r"""Powers of the controlled-Y gate
 
     Locally equivalent to ``Can(t/2, 0, 0)``
 
     """
-    _diagram_labels = [CTRL, "Y^{t}"]
+    cv_target = YPow
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        (t,) = self.params
-        return CY(*self.qubits).hamiltonian * t
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        ctheta = np.pi * var.asfloat(self.param("t"))
-        phase = np.exp(0.5j * ctheta)
-        unitary = [
-            [1, 0, 0, 0],
-            [0, 1, 0, 0],
-            [0, 0, phase * np.cos(ctheta / 2.0), phase * -np.sin(ctheta / 2.0)],
-            [0, 0, phase * np.sin(ctheta / 2.0), phase * np.cos(ctheta / 2.0)],
-        ]
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CYPow":
         return self ** -1
@@ -494,37 +507,18 @@ class CYPow(StdGate):
         (t,) = self.params
         return CYPow(e * t, *self.qubits)
 
-    def decompose(self) -> Iterator[Union[S, S_H, CNotPow]]:
-        """Convert powers of CZ gate to powers of CNOT gate"""
-        (t,) = self.params
-        q0, q1 = self.qubits
-        yield S_H(q1)
-        yield CNot(q0, q1) ** t
-        yield S(q1)
-
 
 # End class CYPow
 
 
-class CV(StdGate):
+class CV(StdCtrlGate):
     r"""A controlled V (sqrt of CNOT) gate."""
 
-    _diagram_labels = [CTRL, "V"]
+    cv_target = V
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        return CNot(*self.qubits).hamiltonian / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        q0, q1 = self.qubits
-        from ..modules import ControlGate
-
-        return ControlGate([q0], V(q1)).tensor
-
     @property
     def H(self) -> "CV_H":
         return CV_H(*self.qubits)
@@ -536,25 +530,14 @@ class CV(StdGate):
 # end class CV
 
 
-class CV_H(StdGate):
+class CV_H(StdCtrlGate):
     r"""A controlled V (sqrt of CNOT) gate."""
 
-    _diagram_labels = [CTRL, "V" + CONJ]
+    cv_target = V_H
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        return -CNot(*self.qubits).hamiltonian / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        q0, q1 = self.qubits
-        from ..modules import ControlGate
-
-        return ControlGate([q0], V_H(q1)).tensor
-
     @property
     def H(self) -> "CV":
         return CV(*self.qubits)
@@ -566,7 +549,7 @@ class CV_H(StdGate):
 # end class CV_H
 
 
-class CZ(StdGate):
+class CZ(StdCtrlGate):
     r"""A controlled-Z gate
 
     Equivalent to ``controlled_gate(Z())`` and locally equivalent to
@@ -576,23 +559,14 @@ class CZ(StdGate):
         \text{CZ}() = \begin{pmatrix} 1&0&0&0 \\ 0&1&0&0 \\
                                     0&0&1&0 \\ 0&0&0&-1 \end{pmatrix}
     """
+    cv_target = Z
+    cv_hermitian = True
     cv_interchangeable = True
     cv_tensor_structure = "diagonal"
-    _diagram_labels = [CTRL, CTRL]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        q0, q1 = self.qubits
-        return Z(q1).hamiltonian * (1 - sZ(q0)) / 2
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        unitary = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, -1]]
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CZ":
         return self  # Hermitian
@@ -607,11 +581,14 @@ class CZ(StdGate):
         tensor[s11] *= -1
         return State(tensor, ket.qubits, ket.memory)
 
+    def _diagram_labels_(self) -> List[str]:
+        return [CTRL, CTRL]
+
 
 # End class CZ
 
 
-class CZPow(StdGate):
+class CZPow(StdCtrlGate):
     r"""Powers of the controlled-Z gate
 
     Locally equivalent to ``Can(t/2, 0, 0)``
@@ -624,29 +601,13 @@ class CZPow(StdGate):
                         0 & 0 & 0 & \exp(i \pi t)
                       \end{pmatrix}
     """
+    cv_target = ZPow
     cv_interchangeable = True
     cv_tensor_structure = "diagonal"
-    _diagram_labels = [CTRL, "Z^{t}"]
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
 
-    @property
-    def hamiltonian(self) -> Pauli:
-        (t,) = self.params
-        return CZ(*self.qubits).hamiltonian * t
-
-    @cached_property
-    def tensor(self) -> QubitTensor:
-        t = var.asfloat(self.param("t"))
-        unitary = [
-            [1, 0, 0, 0],
-            [0, 1, 0, 0],
-            [0, 0, 1, 0],
-            [0, 0, 0, np.exp(1j * t * np.pi)],
-        ]
-        return tensors.asqutensor(unitary)
-
     @property
     def H(self) -> "CZPow":
         return self ** -1
@@ -693,7 +654,7 @@ class ECP(StdGate):
 
 
 class Exch(StdGate):
-    r"""A 2-qubit parametric gate generated from an exchange interaction.
+    r"""A 2-qubit parametric gate generated from an isotropic exchange interaction.
 
     Equivalent to Can(t,t,t)
 
@@ -802,7 +763,6 @@ class ISwap(StdGate):
     """
     cv_interchangeable = True
     cv_tensor_structure = "monomial"
-    _diagram_labels = ["iSwap"]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -844,7 +804,6 @@ class SqrtISwap(StdGate):
     Equivalent to ``Can(-1/4,-1/4,0)``.
     """
     cv_interchangeable = True
-    _diagram_labels = [SQRT + "iSwap"]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -875,7 +834,6 @@ class SqrtISwap_H(StdGate):
     Equivalent to ``Can(1/4, 1/4, 0)``.
     """
     cv_interchangeable = True
-    _diagram_labels = [SQRT + "iSwap" + CONJ]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -906,7 +864,6 @@ class SqrtSwap(StdGate):
     Equivalent to ``Can(1/4, 1/4, 1/4)``.
     """
     cv_interchangeable = True
-    _diagram_labels = [SQRT + "Swap"]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -938,7 +895,6 @@ class SqrtSwap_H(StdGate):
     ``Can(3/4, 1/4, 1/4)``
     """
     cv_interchangeable = True
-    _diagram_labels = [SQRT + "Swap" + CONJ]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -975,8 +931,8 @@ class Swap(StdGate):
 
     """
     cv_interchangeable = True
+    cv_hermitian = True
     cv_tensor_structure = "permutation"
-    _diagram_labels = [SWAP_TARGET, SWAP_TARGET]
 
     def __init__(self, q0: Qubit, q1: Qubit) -> None:
         super().__init__(qubits=[q0, q1])
@@ -1007,6 +963,9 @@ class Swap(StdGate):
         tensor = np.transpose(ket.tensor, perm)
         return State(tensor, ket.qubits, ket.memory)
 
+    def _diagram_labels_(self) -> List[str]:
+        return [SWAP_TARGET] * 2
+
 
 # end class SWAP
 
@@ -1075,7 +1034,6 @@ class XX(StdGate):
     """
     # TODO: Is XX(1/2) MS gate, or is it XX(-1/2)???
     cv_interchangeable = True
-    _cv_sdiagram_labels = ["XX^{t}"]
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
@@ -1113,14 +1071,17 @@ class XX(StdGate):
             return I(qbs[0])
         return self
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["XX^{t}"] * 2
+
 
 class XY(StdGate):
     r"""XY interaction gate.
 
     Powers of the iSWAP gate. Equivalent to ``Can(t, t, 0)``.
     """
+    # https://arxiv.org/abs/1912.04424v1
     cv_interchangeable = True
-    _diagram_labels = ["XY^{t}"]
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
@@ -1144,6 +1105,9 @@ class XY(StdGate):
         (t,) = self.params
         return XY(e * t, *self.qubits)
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["XY^{t}"] * 2
+
 
 class YY(StdGate):
     r"""A parametric 2-qubit gate generated from a YY interaction.
@@ -1155,7 +1119,6 @@ class YY(StdGate):
         t:
     """
     cv_interchangeable = True
-    _diagram_labels = ["YY^{t}"]
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
@@ -1193,6 +1156,9 @@ class YY(StdGate):
             return I(qbs[0])
         return self
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["YY^{t}"] * 2
+
 
 class ZZ(StdGate):
     r"""A parametric 2-qubit gate generated from a ZZ interaction.
@@ -1205,7 +1171,6 @@ class ZZ(StdGate):
     """
     cv_interchangeable = True
     cv_tensor_structure = "diagonal"
-    _diagram_labels = ["ZZ^{t}"]
 
     def __init__(self, t: Variable, q0: Qubit, q1: Qubit) -> None:
         super().__init__(params=[t], qubits=[q0, q1])
@@ -1250,6 +1215,9 @@ class ZZ(StdGate):
             return I(qbs[0])
         return self
 
+    def _diagram_labels_(self) -> List[str]:
+        return ["ZZ^{t}"] * 2
+
 
 # end class ZZ
 
