@@ -22,6 +22,8 @@ def test_ccnot() -> None:
     ket = qf.CCNot(0, 1, 2).run(ket)
     assert ket.tensor[1, 1, 1] == 1.0
 
+    assert qf.CCNot(0, 1, 2).target_qubit_nb == 1
+
 
 def test_cswap() -> None:
     ket = qf.zero_state(3)
@@ -33,6 +35,8 @@ def test_cswap() -> None:
     ket = qf.CSwap(0, 1, 2).run(ket)
     assert ket.tensor[1, 0, 1] == 1.0
 
+    assert qf.CSwap(0, 1, 2).target_qubit_nb == 2
+
 
 def test_ccz() -> None:
     ket = qf.zero_state(3)
@@ -44,7 +48,7 @@ def test_ccz() -> None:
     qf.print_state(ket)
     assert np.isclose(ket.tensor[1, 1, 1], 1.0)
 
-    gate0 = qf.CCZ()
+    gate0 = qf.CCZ(0, 1, 2)
     assert gate0.H is gate0
 
 
