@@ -6,12 +6,15 @@
 # DOCME
 
 import string
-from typing import List, Sequence, Tuple
+from typing import TYPE_CHECKING, List, Sequence, Tuple
 
 import numpy as np
-from numpy.typing import ArrayLike
 
-__all__ = ("QubitTensor", "TensorLike", "asqutensor")
+if TYPE_CHECKING:
+    from numpy.typing import ArrayLike  # pragma: no cover
+
+
+__all__ = ("QubitTensor", "asqutensor")
 
 
 qubit_dtype = np.complex128
@@ -22,11 +25,7 @@ QubitTensor = np.ndarray
 QubitTensor arrays have complex data type, and all axes have length 2."""
 
 
-TensorLike = ArrayLike
-"""Any python object that can be converted into a numpy array"""
-
-
-def asqutensor(array: TensorLike) -> QubitTensor:
+def asqutensor(array: "ArrayLike") -> QubitTensor:
     """Converts a tensor like object to a numpy array object with complex data
     type, and reshapes to [2]*N (So the number of elements must be a power of 2)
     """
@@ -162,7 +161,7 @@ def partial_trace(
 
     subscripts = list(EINSUM_SUBSCRIPTS[0 : N * R])
 
-    print(indices, R, N, subscripts)
+    # print(indices, R, N, subscripts)
     for idx in indices:
         for r in range(1, R):
             subscripts[r * N + idx] = subscripts[idx]
@@ -176,10 +175,7 @@ def partial_trace(
 
 # DOCME
 def tensormul(
-    tensor0: QubitTensor,
-    tensor1: QubitTensor,
-    indices: Tuple[int, ...],
-    diagonal: bool = False,
+    tensor0: QubitTensor, tensor1: QubitTensor, indices: Tuple[int, ...]
 ) -> QubitTensor:
     N = np.ndim(tensor1)
     K = np.ndim(tensor0) // 2
@@ -194,12 +190,34 @@ def tensormul(
     tensor = np.transpose(tensor, perm)
     tensor = np.reshape(tensor, [2 ** K, 2 ** (N - K)])
 
-    if diagonal:
-        tensor = np.transpose(tensor)
-        tensor = tensor * np.diag(gate)
-        tensor = np.transpose(tensor)
-    else:
-        tensor = np.matmul(gate, tensor)
+    tensor = np.matmul(gate, tensor)
+
+    tensor = np.reshape(tensor, [2] * N)
+    tensor = np.transpose(tensor, inv_perm)
+
+    return tensor
+
+
+# DOCME
+def tensormul_diagonal(
+    tensor0_diagonal: QubitTensor,
+    tensor1: QubitTensor,
+    indices: Tuple[int, ...],
+) -> QubitTensor:
+    N = np.ndim(tensor1)
+    K = np.ndim(tensor0_diagonal)
+    assert K == len(indices)
+
+    perm = list(indices) + [n for n in range(N) if n not in indices]
+    inv_perm = np.argsort(perm)
+
+    tensor = tensor1
+    tensor = np.transpose(tensor, perm)
+    tensor = np.reshape(tensor, [2 ** K, 2 ** (N - K)])
+
+    tensor = np.transpose(tensor)
+    tensor = tensor * tensor0_diagonal.flatten()
+    tensor = np.transpose(tensor)
 
     tensor = np.reshape(tensor, [2] * N)
     tensor = np.transpose(tensor, inv_perm)
