@@ -7,9 +7,9 @@
 Useful routines not necessarily intended to be part of the public API.
 """
 
-import functools
 import warnings
 from fractions import Fraction
+from functools import wraps
 from typing import (
     Any,
     Callable,
@@ -32,6 +32,7 @@ __all__ = (
     # Future
     "importlib_metadata",
     "cached_property",
+    "Protocol",
     "deprecated",
     # Collections
     "multi_slice",
@@ -85,12 +86,20 @@ except ImportError:  # pragma: no cover
         return property(wrapper)
 
 
+try:
+    # python >= 3.8
+    from typing import Protocol  # type: ignore
+except ImportError:  # pragma: no cover
+    # python == 3.7
+    from typing_extensions import Protocol  # type: ignore  # noqa: F401
+
+
 def deprecated(func: Callable) -> Callable:
     """This is a decorator which can be used to mark functions
     as deprecated. It will result in a warning being emitted
     when the function is used."""
 
-    @functools.wraps(func)
+    @wraps(func)
     def _new_func(*args: Any, **kwargs: Any) -> Any:
         warnings.simplefilter("always", DeprecationWarning)  # turn off filter
         warnings.warn(
@@ -204,7 +213,12 @@ def bitlist_to_int(bitlist: Sequence[int]) -> int:
     >>> bitlist_to_int([1, 0, 0])
     4
     """
-    return int("".join([str(d) for d in bitlist]), 2)
+    # return int("".join([str(d) for d in bitlist]), 2)
+
+    out = 0
+    for bit in bitlist:
+        out = (out << 1) | bit
+    return out
 
 
 def int_to_bitlist(x: int, pad: int = None) -> List[int]:
