@@ -12,8 +12,9 @@ import os
 import re
 import subprocess
 import tempfile
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Sequence, TextIO
 
+import numpy as np
 import sympy
 from PIL import Image
 from sympy import Symbol
@@ -21,12 +22,11 @@ from sympy import Symbol
 from . import utils, var
 from .circuits import Circuit
 from .dagcircuit import DAGCircuit
-from .gates import P0, P1
-from .modules import IdentityGate
+from .gates import ControlGate, IdentityGate
 from .ops import Gate, Operation
 from .qubits import Qubits
-from .stdgates import CZ, CSwap, Swap
-from .stdops import Reset
+from .stdgates import CZ, CSwap, Swap, X
+from .stdops import Project0, Project1, Reset
 
 __all__ = (
     "LATEX_GATESET",
@@ -34,10 +34,11 @@ __all__ = (
     "latex_to_image",
     "circuit_to_image",
     "circuit_to_diagram",
+    "print_gate",
 )
 
 
-# TODO: Should be set of types to match GATESET in stdgates?
+# FIXME: Use gateset.LATEX_GATES
 LATEX_GATESET = frozenset(
     [
         "I",
@@ -66,6 +67,7 @@ LATEX_GATESET = frozenset(
         "ISwap",
         "PSwap",
         "CV",
+        "CS",
         "CV_H",
         "CPhase",
         "CH",
@@ -80,8 +82,8 @@ LATEX_GATESET = frozenset(
         "YY",
         "ZZ",
         "Can",
-        "P0",
-        "P1",
+        "Project0",
+        "Project1",
         "Reset",
         "NoWire",
         "Measure",
@@ -95,6 +97,9 @@ LATEX_GATESET = frozenset(
         "B",
         "CY",
         "ECP",
+        "Sycamore",
+        "XY",
+        "ControlGate",
     ]
 )
 
@@ -125,7 +130,8 @@ kwarg_to_symbol = {
 class NoWire(IdentityGate):
     """Dummy gate used to draw a gap in a circuit"""
 
-    _diagram_labels = ["  "]
+    def _diagram_labels_(self) -> List[str]:
+        return ["  "]
 
 
 def circuit_to_latex(
@@ -136,13 +142,12 @@ def circuit_to_latex(
     options: str = None,
     scale: float = 0.75,
     qubit_labels: bool = True,
+    left_labels: Sequence[str] = None,  # DOCME TESTME
+    right_labels: Sequence[str] = None,  # DOCME TESTME
 ) -> str:
     """
     Create an image of a quantum circuit in LaTeX.
 
-    Can currently draw X, Y, Z, H, T, S, T_H, S_H, Rx, Ry, Rz, XPow, YPow, ZPow,
-    HPow, CNot, CZ, Swap, ISwap, CCNot, CSwap, XX, YY, ZZ, Can, P0 and P1 gates,
-    and the Reset operation.
 
     Args:
         circ:       A quantum Circuit
@@ -188,6 +193,7 @@ def circuit_to_latex(
         "XX": [r"X\!X^{{{t}}}"],
         "YY": [r"Y\!Y^{{{t}}}"],
         "ZZ": [r"Z\!Z^{{{t}}}"],
+        "XY": [r"X\!Y^{{{t}}}"],
         "ISwap": [r"\text{{iSwap}}"],
         "B": [r"\text{{B}}"],
         "SqrtISwap": [r"\sqrt{{\text{{iSwap}}}}"],
@@ -195,6 +201,8 @@ def circuit_to_latex(
         "CNot": [CTRL, TARGET],
         "CH": [CTRL, "H"],
         "CV": [CTRL, "V"],
+        "CS": [CTRL, "S"],
+        "CPhase": [CTRL, r"P({theta})"],
         "CV_H": [CTRL, r"V^\dagger"],
         "CNotPow": [CTRL, r"X^{{{t}}}"],
         "CY": [CTRL, r"Y"],
@@ -225,6 +233,10 @@ def circuit_to_latex(
         code = [r"\lstick{%s}" % q for q in qubits]
         layer_code.append(code)
 
+    if left_labels:
+        code = [r"\lstick{%s}" % L for L in left_labels]
+        layer_code.append(code)
+
     for layer in layers:
         code = [r"\qw"] * N
 
@@ -276,9 +288,9 @@ def circuit_to_latex(
                 else:  # quantikz
                     for i in idx:
                         code[i] = r""
-            elif isinstance(gate, P0):
+            elif isinstance(gate, Project0):
                 code[idx[0]] = r"\push{\ket{0}\!\!\bra{0}} \qw"
-            elif isinstance(gate, P1):
+            elif isinstance(gate, Project1):
                 code[idx[0]] = r"\push{\ket{1}\!\!\bra{1}} \qw"
             # elif isinstance(gate, Measure):
             #     code[idx[0]] = r"\meter{}"  # TODO: Add cbit label        # FIXME
@@ -299,8 +311,14 @@ def circuit_to_latex(
             elif (
                 gate.qubit_nb == 2
                 and gate.cv_interchangeable
-                and gate.name != "CZPow"
+                and gate.name
+                not in [
+                    "CZPow",
+                    "CPhase",
+                    "CS",
+                ]  # FIXME better approach to selecting correct gates here
                 # and gate.name not in latex_labels
+                # and (gate.name in latex_labels and len(latex_labels[gate.name]) != 1)
             ):
                 top = min(idx)
                 bot = max(idx)
@@ -357,6 +375,15 @@ def circuit_to_latex(
             elif gate.qubit_nb == 1:
                 code[idx[0]] = r"\gate{" + text_labels[0] + "}"
 
+            # DOCME TESTME
+            # TODO: Currently only implemented for multi-controlled X gates
+            # TODO: axes
+            elif isinstance(gate, ControlGate):
+                assert isinstance(gate.target, X)
+                for i in range(len(gate.control_qubits)):
+                    code[idx[i]] = r"\ctrl{" + str(idx[i + 1] - idx[i]) + "}"
+                code[idx[i + 1]] = r"\targ{}"
+
             else:
                 raise NotImplementedError(str(gate))
 
@@ -365,6 +392,10 @@ def circuit_to_latex(
     code = [r"\qw"] * N
     layer_code.append(code)
 
+    if right_labels:
+        code = [r"\rstick{%s}" % L for L in right_labels]
+        layer_code.append(code)
+
     latex_lines = [""] * N
 
     for line, wire in enumerate(zip(*layer_code)):
@@ -428,9 +459,10 @@ QUANTIKZ_FOOTER_ = r"""
 
 
 # TODO: Handle unicode via xelatex?
+# TODO: Use psflatex python package
 
 
-def latex_to_image(latex: str) -> Image:  # pragma: no cover
+def latex_to_image(latex: str) -> Image:
     """
     Convert a single page LaTeX document into an image.
 
@@ -475,7 +507,7 @@ def latex_to_image(latex: str) -> Image:  # pragma: no cover
     return img
 
 
-def circuit_to_image(circ: Circuit, qubits: Qubits = None) -> Image:  # pragma: no cover
+def circuit_to_image(circ: Circuit, qubits: Qubits = None) -> Image:
     """Create an image of a quantum circuit.
 
     A convenience function that calls circuit_to_latex() and latex_to_image().
@@ -536,7 +568,6 @@ def circuit_to_diagram(
         for i in range(i0 + 2, i1, 2):
             code[i] = (BOX_CHARS[LEFT + RIGHT] * left_pad) + BOX_CHARS[CROSS]
 
-    print("Here len", len(circ))
     if len(circ) == 0:
         # Empty circuit
         circ = Circuit(NoWire([0]))
@@ -544,11 +575,9 @@ def circuit_to_diagram(
     if qubits is None:
         qubits = circ.qubits
     N = len(qubits)
-    # print("Here N", N)
 
     qubit_idx = dict(zip(qubits, range(0, 2 * N - 1, 2)))
     layers = _display_layers(circ, qubits)
-    # print(layers)
     layer_text = []
 
     qubit_layer = [""] * (2 * N - 1)
@@ -574,28 +603,21 @@ def circuit_to_diagram(
                     for key, value in zip(elem.cv_args, params)
                 }
 
-            # Construct text labels
-            name = elem.name
-            if elem._diagram_labels:
-                text_labels = elem._diagram_labels
-                if len(idx) != 1 and len(text_labels) == 1:
-                    text_labels = list(text_labels) * len(idx)
-                text_labels = [t.format(**pretty_params) for t in text_labels]
-            else:
-                if pretty_params:
-                    params_text = ",".join(pretty_params.values())
-                    text_labels = [name + "(%s)" % params_text] * len(idx)
-                else:
-                    text_labels = [name] * len(idx)
-                if len(idx) != 1 and not elem.cv_interchangeable:
-                    # If not interchangeable, we have to label connections
-                    for i in range(elem.qubit_nb):
-                        text_labels[i] = text_labels[i] + "_%s" % i
+            text_labels = elem._diagram_labels_()
+
+            vertical_lines = len(text_labels) > 1
+
+            if len(text_labels) == 1 and len(idx) > 1:
+                text_labels *= len(idx)
+
+            assert len(text_labels) == len(idx)
+
+            text_labels = [t.format(**pretty_params) for t in text_labels]
 
             if not use_unicode:
                 text_labels = [_unicode_to_ascii(tl) for tl in text_labels]
 
-            if elem.qubit_nb != 1 and not elem._diagram_noline:
+            if vertical_lines:
                 pad = len(re.split(r"[_^(]+", text_labels[0])[0]) // 2
                 draw_line(code, min(idx), max(idx), left_pad=pad)
 
@@ -669,13 +691,29 @@ def _display_layers(circ: Circuit, qubits: Qubits) -> Circuit:
     return Circuit([Circuit(L) for L in layers])
 
 
+# https://stackoverflow.com/questions/48491577/printing-the-output-rounded-to-3-decimals-in-sympy
+def _round_sympy_expr(expr: sympy.Expr, num_digits: int = 4) -> sympy.Expr:
+    return expr
+    # BROKEN. Undoes rationalizations
+    # return expr.xreplace({n: round(n, num_digits) for n in expr.atoms(sympy.Number)})
+    # return expr.xreplace({n : round(n, num_digits) for n in expr.atoms(sympy.Number)})
+
+
+# FIXME: Refactor
 def _pretty(obj: Any, format: str = "text") -> str:
     """Pretty format an object as a text or latex string."""
     assert format in ["latex", "text"]
+
+    if isinstance(obj, sympy.Expr):
+        if format == "latex":
+            return sympy.latex(_round_sympy_expr((obj)))  # pragma: no cover  # FIXME
+        else:
+            return str(obj)
+
     if isinstance(obj, float):
         try:
             if format == "latex":
-                return sympy.latex(var.asexpression(obj))
+                return sympy.latex(_round_sympy_expr(var.asexpression(obj)))
             else:
                 return str(var.asexpression(obj)).replace("pi", "π")
         except ValueError:
@@ -708,6 +746,7 @@ def _box_char_transpose(chars: str) -> str:
     )
 
 
+# TODO: extra control character
 # FIXME: pi, alpha, ect...
 unicode_ascii = {
     "●": "@",
@@ -724,4 +763,24 @@ def _unicode_to_ascii(text: str) -> str:
     return "".join(unicode_ascii.get(c, c) for c in text)
 
 
+def print_gate(gate: Gate, ndigits: int = 2, file: TextIO = None) -> None:
+    """Pretty print a gate tensor
+
+    Args:
+        gate:
+        ndigits:
+        file: Stream to which to write. Defaults to stdout
+    """
+    N = gate.qubit_nb
+    gate_tensor = gate.tensor
+    lines = []
+    for index, amplitude in np.ndenumerate(gate_tensor):
+        ket = "".join([str(n) for n in index[0:N]])
+        bra = "".join([str(index[n]) for n in range(N, 2 * N)])
+        if round(abs(amplitude) ** 2, ndigits) > 0.0:
+            lines.append(f"{bra} -> {ket} : {amplitude}")
+    lines.sort(key=lambda x: int(x[0:N]))
+    print("\n".join(lines), file=file)
+
+
 # fin
