@@ -8,6 +8,7 @@
 Unit tests for quantumflow.visualization
 """
 
+import io
 import os
 import shutil
 from math import pi
@@ -64,7 +65,7 @@ def test_visualize_circuit() -> None:
 
     circ += qf.CNot(1, 2)
     circ += qf.CNot(2, 1)
-    # circ += qf.IDEN(*range(8))
+
     circ += qf.ISwap(4, 2)
     circ += qf.ISwap(6, 5)
     circ += qf.CZ(1, 3)
@@ -75,8 +76,8 @@ def test_visualize_circuit() -> None:
     circ += qf.CCNot(1, 2, 3)
     circ += qf.CSwap(4, 5, 6)
 
-    circ += qf.P0(0)
-    circ += qf.P1(1)
+    circ += qf.Project0(0)
+    circ += qf.Project1(1)
 
     circ += qf.Reset(2)
     circ += qf.Reset(4, 5, 6)
@@ -105,7 +106,6 @@ def test_visualize_circuit() -> None:
     circ += qf.CH(1, 6)
 
     circ += qf.visualization.NoWire([0, 1, 2])
-    # circ += qf.visualization.NoWire(4, 1, 2)
 
     if os.environ.get("QF_VIZTEST"):
         print()
@@ -121,7 +121,7 @@ def test_visualize_circuit() -> None:
     qf.circuit_to_diagram(circ, use_unicode=False)
 
     latex = qf.circuit_to_latex(circ, package="qcircuit")
-    print(latex)
+
     if os.environ.get("QF_VIZTEST"):
         qf.latex_to_image(latex).show()
 
@@ -133,8 +133,19 @@ def test_visualize_circuit() -> None:
 
 
 def test_circuit_diagram() -> None:
-    circ = qf.Circuit()
-    circ += qf.Givens(pi, 1, 0)  # Not yet supported in latex
+    # Test diagrams creation for a few operations and gates not
+    # necessarily supported in latex
+    circ = qf.Circuit(
+        qf.Givens(pi, 1, 0),
+        qf.IdentityGate([0, 3, 4]),
+        qf.Measure(3, 3),
+        qf.Measure(4),
+        qf.Barrier(0, 2, 3, 4),
+        qf.QFSimulator(qf.Circuit(qf.X(0), qf.X(2))),
+        #  qf.Circuit(qf.X(3), qf.X(4), qf.CNot(3, 4)), FIXME does not work
+        qubits=[0, 1, 2, 3, 4],
+    )
+
     diag = qf.circuit_to_diagram(circ)
     print()
     print(diag)
@@ -142,12 +153,12 @@ def test_circuit_diagram() -> None:
 
 
 @skip_unless_pdflatex
-@pytest.mark.parametrize("gatename", qf.StdGate.cv_stdgates.keys())
+@pytest.mark.parametrize("gatename", qf.STDGATES.keys())
 def test_stdgates_latex_to_image(gatename: str) -> None:
     if gatename not in qf.LATEX_GATESET:
         print("Latex not supported:", gatename)
         return
-    gatet = qf.StdGate.cv_stdgates[gatename]
+    gatet = qf.STDGATES[gatename]
     gate = _randomize_gate(gatet)
     circ = qf.Circuit(gate)
     latex = qf.circuit_to_latex(circ)
@@ -156,6 +167,17 @@ def test_stdgates_latex_to_image(gatename: str) -> None:
         img.show()
 
 
+@pytest.mark.parametrize("gatename", qf.STDGATES.keys())
+def test_circuit_to_diagram_stdgates(gatename: str) -> None:
+    gatet = qf.STDGATES[gatename]
+    gate = _randomize_gate(gatet)
+    circ = qf.Circuit(gate)
+    diag = qf.circuit_to_diagram(circ)
+    print(gatename)
+    print(diag)
+    print()
+
+
 @skip_unless_pdflatex
 def test_latex_to_image() -> None:
     # TODO: Double check this circuit is correct
@@ -440,4 +462,42 @@ def test_gate3_to_diagrams() -> None:
         qf.latex_to_image(latex).show()
 
 
+def test_ControlGate_to_latex() -> None:
+    circ = qf.Circuit()
+    circ += qf.ControlGate(qf.X(3), [0, 1, 2])
+
+    latex = qf.circuit_to_latex(circ)
+    if os.environ.get("QF_VIZTEST"):
+        qf.latex_to_image(latex).show()
+
+
+def test_latex_labels() -> None:
+    circ = qf.Circuit()
+    circ += qf.CCNot(0, 1, 2)
+
+    latex = qf.circuit_to_latex(
+        circ,
+        qubit_labels=False,
+        left_labels=["c0", "c1", "c2", "t"],
+        right_labels=["this", "that", "other", "nother"],
+    )
+    if os.environ.get("QF_VIZTEST"):
+        qf.latex_to_image(latex).show()
+
+
+def test_print_gate() -> None:
+    stream = io.StringIO()
+
+    qf.print_gate(qf.CNot(0, 1), file=stream)
+    s = stream.getvalue()
+    ref = (
+        "00 -> 00 : (1+0j)\n"
+        + "01 -> 01 : (1+0j)\n"
+        + "10 -> 11 : (1+0j)\n"
+        + "11 -> 10 : (1+0j)\n"
+    )
+
+    assert s == ref
+
+
 # fin
