@@ -15,38 +15,51 @@ Interface between IBM's Qiskit and QuantumFlow
 .. autofunction:: circuit_to_qiskit
 """
 
-import qiskit
+# Note: Installation of qiskit is optional, so we defer import
 
-from .circuits import Circuit
-from .ops import Operation, StdGate
-from .qubits import Qubits
-from .states import State
-from .stdops import If, Initialize
-from .translate import circuit_translate, select_translators
-from .utils import invert_map
-
-# This module imports qiskit, so we do not include it at top level.
-# Must be imported explicitly. e.g.
-# > from quantumflow.xqiskit import qiskit_to_circuit, circuit_to_qiskit
-#
 # Note that QASM specific gates are defined in quantumflow/stdgates/stdgates_qasm.py
-# Conceivable you might want to use those gates in QuantumFlow without loading
+# since you might want to use those gates in QuantumFlow without loading
 # qiskit
 
+
+from typing import TYPE_CHECKING
+
+from .circuits import Circuit
+from .gatesets import QISKIT_GATES
+from .states import State
+from .stdgates import STDGATES
+from .stdops import If, Initialize, Simulator
+from .translate import circuit_translate
+from .utils import invert_map
+
+if TYPE_CHECKING:
+    import qiskit  # pragma: no cover
+
+
 __all__ = [
+    "QISKIT_GATES",
     "QiskitSimulator",
     "qiskit_to_circuit",
     "circuit_to_qiskit",
-    "translate_gates_to_qiskit",
+    "translate_to_qiskit",
+    "circuit_to_qasm",
+    "qasm_to_circuit",
+    "translate_gates_to_qiskit",  # Deprecated
 ]
 
+_IMPORT_ERROR_MSG = """External dependency 'qiskit' not installed. Install
+with 'pip install qiskit'"""
+
+
 QASM_TO_QF = {
     "ccx": "CCNot",
     "ch": "CH",
-    "crz": "CRZ",
+    "crz": "CRz",
     "cswap": "CSwap",
-    "cu1": "CPhase",
+    "cu1": "CPhase",  # Legacy. Changed to 'cp'
+    "cp": "CPhase",
     "cu3": "CU3",
+    "csx": "CV",
     "cx": "CNot",
     "cy": "CY",
     "cz": "CZ",
@@ -55,15 +68,19 @@ QASM_TO_QF = {
     "rx": "Rx",
     "ry": "Ry",
     "rz": "Rz",
-    "rzz": "RZZ",
+    "rzz": "Rzz",
     "s": "S",
     "sdg": "S_H",
     "swap": "Swap",
+    "sx": "V",
+    # "sxdg": "V_H",   # Not fully supported by Aer simulator
     "t": "T",
     "tdg": "T_H",
-    "u1": "PhaseShift",
+    "u1": "PhaseShift",  # Legacy. Changed to 'p'
+    "p": "PhaseShift",
     "u2": "U2",
-    "u3": "U3",
+    "u3": "U3",  # Legacy. Changed to 'u'
+    "u": "U3",
     "x": "X",
     "y": "Y",
     "z": "Z",
@@ -74,28 +91,27 @@ QASM_TO_QF = {
 """Map from qiskit operation names to QuantumFlow names"""
 
 
-NAMED_GATES = StdGate.cv_stdgates
-
 # TODO: 'multiplexer', 'snapshot', 'unitary'
 
 
-class QiskitSimulator(Operation):
-    def __init__(self, *elements: Operation) -> None:
-        self._circuit = Circuit(*elements)
-
-    @property
-    def qubits(self) -> Qubits:
-        return self._circuit.qubits
-
+class QiskitSimulator(Simulator):
     def run(self, ket: State = None) -> State:
-        circ = self._circuit
+        try:
+            import qiskit
+        except ModuleNotFoundError as err:  # pragma: no cover
+            raise ModuleNotFoundError(_IMPORT_ERROR_MSG) from err
+
+        circ = self.circuit
         if ket is not None:
             circ = Circuit(Initialize(ket)) + circ
 
         qkcircuit = circuit_to_qiskit(circ, translate=True)
-        simulator = qiskit.Aer.get_backend("statevector_simulator")
-        job = qiskit.execute(qkcircuit, simulator)
-        result = job.result()
+
+        # The call to get_backend() automagically adds the save_statevector() method to
+        # QuantumCircuit. WTF. Mutating classes!? This is a terrible design.
+        simulator = qiskit.Aer.get_backend("aer_simulator")
+        qkcircuit.save_statevector()
+        result = simulator.run(qkcircuit).result()
         tensor = result.get_statevector()
 
         res = State(tensor, list(reversed(self.qubits)))
@@ -105,23 +121,24 @@ class QiskitSimulator(Operation):
         return res
 
 
-def qiskit_to_circuit(qkcircuit: qiskit.QuantumCircuit) -> Circuit:
+def qiskit_to_circuit(qkcircuit: "qiskit.QuantumCircuit") -> Circuit:
     """Convert a qsikit QuantumCircuit to QuantumFlow's Circuit"""
     # We assume that there is only one quantum register of qubits.
 
-    named_ops = dict(StdGate.cv_stdgates)
-
     circ = Circuit()
 
+    qkqbs = qkcircuit.qregs[0][:]
+
     for instruction, qargs, cargs in qkcircuit:
         name = instruction.name
         if name not in QASM_TO_QF:
             raise NotImplementedError("Unknown qiskit operation")
 
         qf_name = QASM_TO_QF[name]
-        qubits = [q.index for q in qargs]
+        qubits = [qkqbs.index(q) for q in qargs]
+
         args = [float(param) for param in instruction.params] + qubits
-        gate = named_ops[qf_name](*args)  # type: ignore
+        gate = STDGATES[qf_name](*args)  # type: ignore
 
         if instruction.condition is None:
             circ += gate
@@ -132,9 +149,9 @@ def qiskit_to_circuit(qkcircuit: qiskit.QuantumCircuit) -> Circuit:
     return circ
 
 
-# TODO: Default translate to false?
-# TODO: QISKIT_GATES
-def circuit_to_qiskit(circ: Circuit, translate: bool = True) -> qiskit.QuantumCircuit:
+def circuit_to_qiskit(
+    circ: Circuit, translate: bool = False
+) -> "qiskit.QuantumCircuit":
     """Convert a QuantumFlow's Circuit to a qsikit QuantumCircuit."""
 
     # In qiskit each gate is defined as a class, and then a method is
@@ -142,11 +159,16 @@ def circuit_to_qiskit(circ: Circuit, translate: bool = True) -> qiskit.QuantumCi
     # append it to the circuit. The method names correspond to the qasm
     # names in QASM_TO_QF
 
-    if translate:
-        circ = translate_gates_to_qiskit(circ)
+    try:
+        import qiskit
+    except ModuleNotFoundError as err:  # pragma: no cover
+        raise ModuleNotFoundError(_IMPORT_ERROR_MSG) from err
 
+    if translate:
+        circ = translate_to_qiskit(circ)
+
+    # Note that with multiple mappings, the second one gets priority.
     QF_TO_QASM = invert_map(QASM_TO_QF)
-    # QF_TO_QASM["I"] = "i"
 
     # We assume only one QuantumRegister. Represent qubits by index in register
     qreg = qiskit.QuantumRegister(circ.qubit_nb)
@@ -174,13 +196,29 @@ def circuit_to_qiskit(circ: Circuit, translate: bool = True) -> qiskit.QuantumCi
     return qkcircuit
 
 
-def translate_gates_to_qiskit(circ: Circuit) -> Circuit:
+def translate_to_qiskit(circ: Circuit) -> Circuit:
     """Convert QF gates to gates understood by qiskit"""
-    target_gates = list([NAMED_GATES[n] for n in QASM_TO_QF.values()])
-    trans = select_translators(target_gates)  # type: ignore
+    return circuit_translate(circ, targets=QISKIT_GATES)
 
-    circ = circuit_translate(circ, trans)
-    return circ
+
+# Deprecated
+translate_gates_to_qiskit = translate_to_qiskit
+
+
+def circuit_to_qasm(circ: Circuit, translate: bool = False) -> str:
+    """Convert a QF circuit to a QASM formatted string"""
+    return circuit_to_qiskit(circ, translate).qasm()
+
+
+def qasm_to_circuit(qasm_str: str) -> Circuit:
+    """Convert a QASM circuit to a QF circuit"""
+    try:
+        import qiskit
+    except ModuleNotFoundError as err:  # pragma: no cover
+        raise ModuleNotFoundError(_IMPORT_ERROR_MSG) from err
+
+    qc = qiskit.QuantumCircuit.from_qasm_str(qasm_str)
+    return qiskit_to_circuit(qc)
 
 
 # fin
